#pip install llama-index
#pip install llama-index-vector-stores-qdrant
#pip install llama-index-embeddings-fastembed
#pip install llama-index-embeddings-langchain
#pip install qdrant-client
#pip install redis
#pip install langchain_huggingface
from qdrant_client import QdrantClient
from qdrant_client.http.models import VectorParams
from llama_index.vector_stores.qdrant import QdrantVectorStore
from llama_index.core import StorageContext, VectorStoreIndex
from llama_index.readers.file.docs.base import PDFReader
from llama_index.core.node_parser import SimpleNodeParser
from llama_index.embeddings.langchain import LangchainEmbedding
from langchain_huggingface import HuggingFaceEmbeddings
import os
import redis



redis_client = redis.Redis(
host='',
 port=6379,
 db=0,
 decode_responses=True)

# Qdrant istemcisi
client = QdrantClient(host="", port=6333)

#Yeni: Koleksiyon kontrol edip sadece yoksa oluştur
def ensure_collection(client, collection_name):
    if client.collection_exists(collection_name):
        return False,f"Koleksiyon zaten var: {collection_name}"
    else:
        client.create_collection(
            collection_name=collection_name,
            vectors_config=VectorParams(size=1024, distance="Cosine")
        )
        return True,f"Yeni koleksiyon oluşturuldu: {collection_name}"

def upload_pdf_to_qdrant(file, collection_name):
    try:
        # 1. Koleksiyonu kontrol edip oluştur
        collections = client.get_collections()
        books = []
        for col in collections.collections:
            print(f"{col.name}")
            redis_key = f"collection:{col.name}"
            credentials = redis_client.hgetall(redis_key)
            print(credentials)
            #if not credentials:
                #print(f'{col.name} icinde kitap yok')
            if credentials:
                book = credentials.get('book')
                books.append({book: col.name})

        if len(books) > 0:
            for x in books:
                if file in x:
                    return f'{file} zaten var collection:{x[file]}'
        else:
            collection_control, explanation = ensure_collection(client, collection_name)
            if collection_control:
                # 2. HuggingFace modelini LlamaIndex'e sarmala
                model_path = "local_bge_model"
                model_folder = os.path.join(os.getcwd(), model_path)
                hf_embed = HuggingFaceEmbeddings(model_name=model_folder)
                embed_model = LangchainEmbedding(hf_embed)

                # 3. PDF oku
                documents = PDFReader().load_data(file)

                # 4. Chunklama
                parser = SimpleNodeParser(chunk_size=512, chunk_overlap=50)

                # 5. Qdrant vector store
                vector_store = QdrantVectorStore(client=client, collection_name=collection_name)
                storage_context = StorageContext.from_defaults(vector_store=vector_store)

                # 6. Embedding’leri Qdrant'a yükle
                VectorStoreIndex.from_documents(
                    documents=documents,
                    storage_context=storage_context,
                    node_parser=parser,
                    embed_model=embed_model
                )
                redis_key = f"collection:{collection_name}"
                redis_client.hset(redis_key, mapping={
                    'collection': collection_name,
                    'book': file
                })

                return f"'{file}' dosyası '{collection_name}' koleksiyonuna yüklendi."
            else:
                return explanation
    except Exception as e:
        return f"Hata: {str(e)}"


# Komut satırından dosya ve koleksiyon al
file = input('file: ')
if file not in os.listdir(os.getcwd()) or not file.endswith('.pdf'):
    print('PDF dosyası gerekli.')
else:
    collection = input('collection: ')
    output = upload_pdf_to_qdrant(file, collection)
    print(output)
